@@ -8,31 +8,28 @@ from numpy import asarray, concatenate, ones
 class Polygon(object):
     # Adapt Shapely or GeoJSON/geo_interface polygons to a common interface
     def __init__(self, context):
-        if hasattr(context, 'interiors'):
-            self.context = context
+        if isinstance(context, dict):
+            self.context = context['coordinates']
         else:
-            self.context = getattr(context, '__geo_interface__', context)
-    @property
-    def geom_type(self):
-        return (getattr(self.context, 'geom_type', None)
-                or self.context['type'])
+            self.context = context
+
     @property
     def exterior(self):
-        return (getattr(self.context, 'exterior', None) 
-                or self.context['coordinates'][0])
+        return (getattr(self.context, 'exterior', None)
+                or self.context[0])
+
     @property
     def interiors(self):
         value = getattr(self.context, 'interiors', None)
         if value is None:
-            value = self.context['coordinates'][1:]
+            value = self.context[1:]
         return value
 
 
 def PolygonPath(polygon):
     """Constructs a compound matplotlib path from a Shapely or GeoJSON-like
     geometric object"""
-    this = Polygon(polygon)
-    assert this.geom_type == 'Polygon'
+
     def coding(ob):
         # The codes will be all "LINETO" commands, except for "MOVETO"s at the
         # beginning of each subpath
@@ -40,18 +37,42 @@ def PolygonPath(polygon):
         vals = ones(n, dtype=Path.code_type) * Path.LINETO
         vals[0] = Path.MOVETO
         return vals
-    vertices = concatenate(
-                    [asarray(this.exterior)[:, :2]] 
-                    + [asarray(r)[:, :2] for r in this.interiors])
-    codes = concatenate(
-                [coding(this.exterior)] 
-                + [coding(r) for r in this.interiors])
+
+    if hasattr(polygon, 'geom_type'):  # Shapely
+        ptype = polygon.geom_type
+        if ptype == 'Polygon':
+            polygon = [Polygon(polygon)]
+        elif ptype == 'MultiPolygon':
+            polygon = [Polygon(p) for p in polygon]
+        else:
+            raise ValueError(
+                "A polygon or multi-polygon representation is required")
+
+    else:  # GeoJSON
+        polygon = getattr(polygon, '__geo_interface__', polygon)
+        ptype = polygon["type"]
+        if ptype == 'Polygon':
+            polygon = [Polygon(polygon)]
+        elif ptype == 'MultiPolygon':
+            polygon = [Polygon(p) for p in polygon['coordinates']]
+        else:
+            raise ValueError(
+                "A polygon or multi-polygon representation is required")
+
+    vertices = concatenate([
+        concatenate([asarray(t.exterior)[:, :2]] +
+                    [asarray(r)[:, :2] for r in t.interiors])
+        for t in polygon])
+    codes = concatenate([
+        concatenate([coding(t.exterior)] +
+                    [coding(r) for r in t.interiors]) for t in polygon])
+
     return Path(vertices, codes)
 
 
 def PolygonPatch(polygon, **kwargs):
     """Constructs a matplotlib patch from a geometric object
-    
+
     The `polygon` may be a Shapely or GeoJSON-like object with or without holes.
     The `kwargs` are those supported by the matplotlib.patches.Polygon class
     constructor. Returns an instance of matplotlib.patches.PathPatch.
